;###########################################################################
;
; LICENSE
;
; Copyright 2004 CAPCA at University of Calgary.
;
; This software is provided "as-is", without any express or
; implied warranty. In no event will the authors be held liable
; for any damages arising from the use of this software.
;
; Permission is granted to anyone to use this software for any
; purpose, including commercial applications, and to alter it and
; redistribute it freely, subject to the following restrictions:
;
; 1. The origin of this software must not be misrepresented; you must
;    not claim you wrote the original software. If you use this software
;    in a product, an acknowledgment in the product documentation
;    would be appreciated, but is not required.
;
; 2. Altered source versions must be plainly marked as such, and must
;    not be misrepresented as being the original software.
;
; 3. This notice may not be removed or altered from any source distribution.
;
;
; The newest version is available from http://www.capca.ucalgary.ca/software
;
;###########################################################################
;
; Jetget-tool to define ones own variables, based on those in the data-file.
;
; Written spring 2003 by Jan Staff.
;
; To be accessed from JETGETs Setup module.
;
; Variables:
;      varlabels  : the variable label
;      variables  : the expression for the variable
;      veclabels  : the vector label if the variable is part of a vector or 0
;      veclabels2 : transformed form of veclabels
;      veccomp    : which vector component the variable is or 0
;      veccont    : matrix containing the expressions for the vector components
;
;    basically, this is how it looks:
;
;    varlabels = (c1, c2, c3, g)
;    variables = (data=b1*10, data=b2*10, data=b3*10, data=e1data*72)
;    veclabels = (c, c, c, 0)
;    veccomp   = (1, 2, 3, 0)
;
;    veccont and veclabels2 comes into play when the variables are sorted
;    different:
;
;    veccont = (data=b1*10, data=b2*10, data=b3*10)
;    veclabels2 = (c)
;
;    so only the vector part is left in these variables, scalars are taken
;    out.
;
;    veccont is a 3xn matrix, n being the number of vectors.
;    veclabels2 is a 1xn matrix.
;
;    veccont and veclabels2 is convenient when using the vectors, eg in
;    fieldlines, whereas the first form is convenient when saving to file in
;    setup, and it is also the way data is read from the widget.
;

pro calculate_variables_event, Event

  common new_variables
  common setupvariables

  widget_control, event.id, get_uvalue = uvalue
  widget_control, event.top, get_uvalue=obj
;
; Which button was pressed:
;
  case uvalue of
     'addbutton'    : obj->run, uvalue, event
     'cancelbutton' : obj->cancel, event
     'Close'        : obj->done, uvalue, event
     'delete'       : obj->delete, event
     else           : obj->tool_event, event
  endcase

end

pro vars::done, uvalue, event
;
; Save and exit
;
  common new_variables
;
; If the array containing the variables (and showing them at the bottom)
; exists:
;
  if widget_info(event.top, find_by_uname="list") ne 0 then begin
   widget_control, widget_info(event.top, find_by_uname="list"), get_value=array
;
; Put each column into its respective variable for later use
;
   varlabels=transpose(array[0,*])
   variables=transpose(array[1,*])
   veclabels2=transpose(array[2,*])
   veccomp=transpose(array[3,*])
  endif else begin
;
; Else (if the array does not exist), make sure the different variables are 
; empty
;
   if n_elements(varlabels) ne 0 then begin
    a=temporary(varlabels)
    a=temporary(variables)
    a=temporary(veclabels2)
    a=temporary(veccomp)
   endif
  endelse
;
; Get the expression for the variable
;
  textfield=widget_info(Self.inbase, find_by_uname='inp')
  widget_control, textfield, get_value=text
  if text ne "" then begin
;
; Put it into its array holding it:
;
   if n_elements(variables) eq 0 then begin
     variables=["data="+text[0]]
   endif else begin
     variables=[variables,"data="+text[0]]
   endelse
;
; Get the label for the first expression, and put it into the array holding
; it
; 
   textlabel=widget_info(Self.inbase, find_by_uname='inpl')
   widget_control, textlabel, get_value=textl
   if n_elements(varlabels) eq 0 then begin
     varlabels=[textl[0]]
   endif else begin
     varlabels=[varlabels, textl[0]]
   endelse
;
; If it is part of a vector, get the information and put that into the
; arrays. First the label
;
   veclabel=widget_info(self.inbase, find_by_uname='vecvar')
   widget_control, veclabel, get_value=vector
   if vector[0] ne "" then begin
    if n_elements(veclabels2) eq 0 then begin
      veclabels2=[vector[0]]
    endif else begin
      veclabels2=[veclabels2, vector[0]]
    endelse
   endif else begin
     if n_elements(veclabels2) eq 0 then begin
       veclabels2=['0']
     endif else begin
       veclabels2=[veclabels2, '0']
     endelse
   endelse
;
; Then the vector component
;
   widget_control, widget_info(self.inbase, find_by_uname='veccomp'), $ 
                 get_value=comp
   if comp[0] ne "" then begin
     if n_elements(veccomp) eq 0 then begin
       veccomp=[comp[0]]
     endif else begin
       veccomp=[veccomp, comp[0]]
     endelse
   endif else begin
     if n_elements(veccomp) eq 0 then begin
       veccomp=['0']
     endif else begin
       veccomp=[veccomp, '0']
     endelse
   endelse
  endif
;
; Update the table showing the variables in the widget
;
  widarr = widget_info(self.inbase, find_by_uname='list')
  if widarr eq 0 then begin
    list = widget_table(self.inbase, /editable, uvalue="list", uname="list", $ 
    value=[transpose(varlabels),transpose(variables),transpose(veclabels2), transpose(veccomp)], $
          column_labels=["Label", "Expression", "Vector", "Component"], $
          /resizeable_columns)
  endif else begin
    widget_control, widarr, insert_rows=1
    widget_control, widarr, set_value=[transpose(varlabels),transpose(variables), transpose(veclabels2), transpose(veccomp)]
  endelse
;
; Read in the same array again. There must be a reason for this, but I have
; luckily forgot what it was.
;
  widget_control, widget_info(event.top, find_by_uname="list"), get_value=array

  varlabels=transpose(array[0,*])
  variables=transpose(array[1,*])
  veclabels=transpose(array[2,*])
  veccomp=transpose(array[3,*])
;
; Make the veccont matrix (see above), and put the quantities into it.
;
  if n_elements(where(veclabels ne '0')) gt 2 then begin
    veccont=strarr(3,n_elements(varlabels)/3)
  endif
  veclabels2=strarr(n_elements(veclabels))
  for i=0,n_elements(veclabels)-1 do begin
    if veclabels[i] ne '0' then begin
      j=where(veclabels2 eq veclabels[i])
      if j[0] ne -1 then begin
        veccont[veccomp[i]-1, j] = variables[i]
      endif else begin
        j=where(veccont[0,*] eq '') 
        veclabels2[j[0]]=veclabels[i]
        veccont[veccomp[i]-1,j[0]]=variables[i]
      endelse
    endif
  endfor

  widget_control, event.top, /destroy

end

pro vars::delete, event
;
; Delete a variable
;
  widget_control, widget_info(event.top, find_by_uname="list"), get_value=tmp
;
; Delete the row from the array showing the variables. The changes will
; take effect when the Done button is pressed.
;
  del_row_table=widget_info(event.top, find_by_uname="list")  

  widget_control, del_row_table, delete_rows=1, USE_TABLE_SELECT=1
  if n_elements(tmp[0,*]) eq 1 then begin
    widget_control, del_row_table, /destroy
    arr = 0
  endif
end

pro vars::cancel, event
;
; The cancel button
;
  common new_variables
;
; Forget everything (that's what it does)
;
  If n_elements(varlabels) ne 0 then begin
    a=temporary(varlabels)
    a=temporary(variables)
    a=temporary(veclabels2)
    a=temporary(veccomp)
  endif

  widget_control, event.top, /destroy
end

pro vars::run, uvalue, ev
;
; Another variable button
;
  common new_variables

  self.arr=1
;
; If the array containing own defined variables exist, read it in and 
; put each column into its own variable
;
  if widget_info(ev.top, find_by_uname="list") ne 0 then begin
    widget_control, widget_info(ev.top, find_by_uname="list"), get_value=array

    varlabels=transpose(array[0,*])
    variables=transpose(array[1,*])
    veclabels2=transpose(array[2,*])
    veccomp=transpose(array[3,*])
  endif else begin
;
; Else make sure those variables are empty
;
    if n_elements(varlabels) ne 0 then begin
      a=temporary(varlabels)
      a=temporary(variables)
      a=temporary(veclabels2)
      a=temporary(veccomp)
    endif
  endelse
;
; Read the expression
; 
  textfield=widget_info(Self.inbase, find_by_uname='inp')
  widget_control, textfield, get_value=text
  if n_elements(variables) eq 0 then begin
    variables=["data="+text[0]]
  endif else begin
    variables=[variables,"data="+text[0]]
  endelse
;
; The label
;
  textlabel=widget_info(Self.inbase, find_by_uname='inpl')
  widget_control, textlabel, get_value=textl
  if n_elements(varlabels) eq 0 then begin
    varlabels=[textl[0]]
  endif else begin
    varlabels=[varlabels, textl[0]]
  endelse
;
; If it is a vector, read that
;
  veclabel=widget_info(self.inbase, find_by_uname='vecvar')
  widget_control, veclabel, get_value=vector
  print, vector
  if vector[0] ne "" then begin
    if n_elements(veclabels2) eq 0 then begin
      veclabels2=[vector[0]]
    endif else begin
      veclabels2=[veclabels2, vector[0]]
    endelse
  endif else begin
    if n_elements(veclabels2) eq 0 then begin
      veclabels2=['0']
    endif else begin
      veclabels2=[veclabels2, '0']
    endelse
  endelse
;
; And the vector component
;
  widget_control, widget_info(self.inbase, find_by_uname='veccomp'), $ 
                get_value=comp
  if comp[0] ne "" then begin
    if n_elements(veccomp) eq 0 then begin
      veccomp=[comp[0]]
    endif else begin
      veccomp=[veccomp, comp[0]]
    endelse
  endif else begin
    if n_elements(veccomp) eq 0 then begin
      veccomp=['0']
    endif else begin
      veccomp=[veccomp, '0']
    endelse
  endelse
;
; Update the array in the bottom of the widget
;
  widarr = widget_info(self.inbase, find_by_uname='list')
;
; If it doesn't exist, make it
;
  if widarr eq 0 then begin
    widget_control, widget_info(self.inbase, find_by_uname='delete'), $
        /destroy
    list = widget_table(self.inbase, /editable, uvalue="list", uname="list", $ 
    value=[transpose(varlabels),transpose(variables),transpose(veclabels2), transpose(veccomp)], $
          column_labels=["Label", "Expression", "Vector", "Component"], $
          /resizeable_columns)
    deletebutton = widget_button(self.inbase, value="Delete a variable", $ 
                             uvalue="delete", uname="delete")
   
  endif else begin
;
; Else update it
; 
    widget_control, widarr, insert_rows=1
    widget_control, widarr, set_value=[transpose(varlabels),transpose(variables), transpose(veclabels2), transpose(veccomp)]
  endelse

  widget_control, textfield, set_value=""
  widget_control, textlabel, set_value=""

end

function vars::init
;
; The init function
;
  common new_variables

  status = self->Tool::Init('Calculate_variables',/nowindow)
;
; The input forms
;
  inp_form = widget_label(self.inbase, value="Add expression (must be in IDL syntax): ")
  inp_form = widget_label(self.inbase, value="Example: b1data^2+b2data^2+b3data^2 ")

  inp_l = cw_field(self.inbase, title="Label:     ", value='', uvalue="inpl",$ 
              uname="inpl")

  inp = cw_field(self.inbase, title='Expression:', value='', uvalue="inp", $
              uname="inp")

  vlabel = widget_label(self.inbase, value="If this variable is part of a vector, then indicate which:")
  vectorvar = cw_field(self.inbase, title='Vector:    ', value="", $ 
              uvalue="vecvar", uname="vecvar")

  veccompwid = cw_field(self.inbase, title='Component: ', value="", $
              uvalue="veccomp", uname="veccomp")
;
; The buttons:
;
  add_button = widget_button(self.inbase, value="Another variable", $
              uvalue="addbutton", uname="addbutton")

  cancel_button = widget_button(self.inbase, value="Cancel", $
              uvalue="cancelbutton", uname="cancelbutton")

  help_button = widget_button(self.inbase, value="Help", uvalue="Help", $
              uname="Help")

  done_button = widget_button(self.inbase, value="Done", $
              uvalue="Close", uname="Close")
;
; The array containing the variables
;
  listbase = widget_base(self.inbase, column=1)
  listlabel = widget_label(listbase, value="Current own defined variables")
  if n_elements(varlabels) gt 0 then begin
    list = widget_table(self.inbase, /editable, uvalue="list", uname="list", $ 
                value=[transpose(varlabels),transpose(variables), transpose(veclabels), transpose(veccomp)], $
                column_labels=["Label", "Expression", "Vector", "Component"], $ 
                /resizeable_columns)
  endif else begin
  endelse
  deletebutton = widget_button(self.inbase, value="Delete a variable", $ 
              uvalue="delete", uname="delete")

  self.helpfile = "help/HELP_VARS"

  widget_control, self.base, /update
  return, 1
end

pro vars__define
   void = {vars, arr:0, INHERITS tool}
end
